from __future__ import annotations

from typing import Literal

import ckan.plugins.toolkit as tk

SELLFINFO_SET_URL = "ckan.selfinfo.page_url"
SELLFINFO_DEFAULT_URL = "/ckan-admin/selfinfo"
SELLFINFO_SET_MAIN_ACTION_NAME = "ckan.selfinfo.main_action_name"
SELLFINFO_DEFAULT_MAIN_ACTION_NAME = "get_selfinfo"
SELFINFO_REDIS_PREFIX = "ckan.selfinfo.redis_prefix_key"
SELFINFO_ERRORS_LIMIT = "ckan.selfinfo.errors_limit"
SELFINFO_REPOS_PATH = "ckan.selfinfo.ckan_repos_path"
SELFINFO_REPOS = "ckan.selfinfo.ckan_repos"
SELFINFO_PARTITIONS_PATH = "ckan.selfinfo.partitions"
SELFINFO_REDIS_SUFFIX: Literal["_selfinfo"] = "_selfinfo"
SELFINFO_CATEGORIES_LIST = "ckan.selfinfo.categories_list"
SELFINFO_ADDITIONAL_PROFILES_USING_REDIS_KEYS = (
    "ckan.selfinfo.additional_profiles_using_redis_keys"
)
STORE_TIME: float = 604800.0  # one week
# STORE_TIME: float = 1.0
PYPI_URL: Literal["https://pypi.org/pypi/"] = "https://pypi.org/pypi/"


def selfinfo_get_path():
    return tk.config.get(SELLFINFO_SET_URL, SELLFINFO_DEFAULT_URL)


def selfinfo_get_main_action_name():
    return tk.config.get(
        SELLFINFO_SET_MAIN_ACTION_NAME, SELLFINFO_DEFAULT_MAIN_ACTION_NAME
    )


def selfinfo_get_redis_prefix():
    prefix = tk.config.get(SELFINFO_REDIS_PREFIX, "")
    return prefix + "_" if prefix else prefix


def selfinfo_get_errors_limit():
    return tk.config.get(SELFINFO_ERRORS_LIMIT)


def selfinfo_get_repos_path():
    return tk.config.get(SELFINFO_REPOS_PATH)


def selfinfo_get_repos():
    return tk.config.get(SELFINFO_REPOS, "")


def selfinfo_get_partitions():
    return tk.config.get(SELFINFO_PARTITIONS_PATH)


def selfinfo_get_categories():
    return tk.config.get(SELFINFO_CATEGORIES_LIST)


def selfinfo_get_additional_redis_keys():
    return tk.config.get(SELFINFO_ADDITIONAL_PROFILES_USING_REDIS_KEYS)



from __future__ import annotations

from typing import Any


from ckan import types
import ckan.plugins.toolkit as tk
import ckan.plugins as p

import ckanext.selfinfo.utils as selfutils
from ckanext.selfinfo.interfaces import ISelfinfo
import ckanext.selfinfo.config as self_config


@tk.side_effect_free
def get_selfinfo(
    context: types.Context,
    data_dict: dict[str, Any],
) -> dict[str, Any]:

    tk.check_access("sysadmin", context, data_dict)

    categories = self_config.selfinfo_get_categories()

    data = {
        "python_modules": selfutils.get_python_modules_info,
        "platform_info": selfutils.get_platform_info,
        "ram_usage": selfutils.get_ram_usage,
        "disk_usage": selfutils.get_disk_usage,
        "git_info": selfutils.gather_git_info,
        "freeze": selfutils.get_freeze,
        "errors": selfutils.retrieve_errors,
        "actions": selfutils.ckan_actions,
        "auth_actions": selfutils.ckan_auth_actions,
        "blueprints": selfutils.ckan_bluprints,
        "helpers": selfutils.ckan_helpers,
        "status_show": selfutils.get_status_show,
    }

    data = {
        key: func() for key, func in data.items() if not categories or key in categories
    }

    # data modification
    for item in p.PluginImplementations(ISelfinfo):
        item.selfinfo_after_prepared(data)

    return data


def selfinfo_get_ram(
    context: types.Context,
    data_dict: dict[str, Any],
) -> dict[str, Any]:

    tk.check_access("sysadmin", context, data_dict)

    return selfutils.get_ram_usage()



from __future__ import annotations

from typing import Any
import pytest
import os

from ckan import model
import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_action
import ckan.tests.factories as factories

import ckanext.selfinfo.config as self_config

current_path: list[str] = os.getcwd().split("/")
current_path.pop()
updated_path: str = "/".join(current_path)


@pytest.mark.ckan_config("ckan.plugins", "selfinfo")
@pytest.mark.usefixtures("with_plugins", "clean_db")
@pytest.mark.ckan_config("ckan.selfinfo.ckan_repos_path", updated_path)
@pytest.mark.ckan_config("ckan.selfinfo.ckan_repos", "ckan ckanext-selfinfo")
class TestUPDATE:
    def test_update_last_module_check(self):
        user = factories.User()
        sysadmin = factories.Sysadmin()
        context: dict[str, Any] = {
            "model": model,
            "user": user["name"],
            "ignore_auth": False,
        }

        with pytest.raises(tk.NotAuthorized):
            call_action(self_config.selfinfo_get_main_action_name(), context=context)

        context["user"] = sysadmin["name"]
        selfinfo: dict[str, Any] = tk.get_action(
            self_config.selfinfo_get_main_action_name()
        )(context, {})
        ckan_info = selfinfo["groups"]["ckan"]["ckan"]

        updated: dict[str, Any] = tk.get_action("update_last_module_check")(
            context, {"module": "ckan"}
        )

        assert ckan_info != updated
        assert ckan_info["updated"] < updated["updated"]



from __future__ import annotations

import time
import json
from typing import Any, cast
from flask import Blueprint, Response
from flask.views import MethodView

from ckan import types
import ckan.model as model
import ckan.plugins.toolkit as tk
from ckan.lib.redis import connect_to_redis, Redis
from ckanext.selfinfo.utils import get_redis_key, retrieve_additionals_redis_keys_info
import ckanext.selfinfo.config as self_config

selfinfo = Blueprint("selfinfo", __name__)


class SelfinfoView(MethodView):
    def get(self):
        try:
            context: types.Context = cast(
                types.Context,
                {
                    "model": model,
                    "user": tk.current_user.name,
                    "auth_user_obj": tk.current_user,
                },
            )

            tk.check_access("sysadmin", context)
        except tk.NotAuthorized:
            tk.abort(404)

        args = tk.request.args

        if args.get("drop_errors") and tk.asbool(args["drop_errors"]):
            redis: Redis = connect_to_redis()
            key = get_redis_key("errors")
            redis.set(key, json.dumps([]))

            return tk.redirect_to("selfinfo.index")

        data: dict[str, Any] = tk.get_action(
            self_config.selfinfo_get_main_action_name()
        )({}, {})

        profiles = {"default": data}
        additional_keys = self_config.selfinfo_get_additional_redis_keys()

        if additional_keys:
            for key in additional_keys:
                profiles[key] = retrieve_additionals_redis_keys_info(key)

        return tk.render(
            "selfinfo/index.html",
            {
                "profiles": profiles,
            },
        )


@selfinfo.route("/selfinfo/get-ram")
def selfinfo_get_ram():
    try:
        context: types.Context = cast(
            types.Context,
            {
                "model": model,
                "user": tk.current_user.name,
                "auth_user_obj": tk.current_user,
            },
        )

        tk.check_access("sysadmin", context)
    except tk.NotAuthorized:
        tk.abort(404)

    ram = tk.get_action("selfinfo_get_ram")(context, {})

    html = "<tr>"
    html += "<td>" + str(ram["precent_usage"]) + "%</td>"
    html += "<td>" + str(ram["used_ram"]) + "</td>"
    html += "<td>" + str(ram["total_ram"]) + "</td>"
    html += "<td><div class=row>"
    html += "<div class='col-lg-6'><ol start=1>"
    for process in ram["processes"][:5]:
        html += "<li>"
        html += "<span title='memory'>" + process[2] + "</span>, "
        html += "<span title='name'>" + process[1] + "</span>, "
        html += "<span title='PID'>" + str(process[0]) + "</span>"
        html += "</li>"
    html += "</div></ol>"
    html += "<div class='col-lg-6'><ol start=6>"
    for process in ram["processes"][5:]:
        html += "<li>"
        html += "<span title='memory'>" + process[2] + "</span>, "
        html += "<span title='name'>" + process[1] + "</span>, "
        html += "<span title='PID'>" + str(process[0]) + "</span>"
        html += "</li>"
    html += "</div></ol>"
    html += "</div></td>"
    html += "</tr>"
    return html


selfinfo.add_url_rule(
    self_config.selfinfo_get_path(),
    view_func=SelfinfoView.as_view("index"),
)



from __future__ import annotations

from typing import Any, Mapping
from datetime import datetime
import importlib_metadata as imetadata

from ckan import types
from ckan.lib.redis import connect_to_redis, Redis
import ckan.plugins.toolkit as tk

import ckanext.selfinfo.utils as selfutils
import ckanext.selfinfo.config as self_config


def update_last_module_check(
    context: types.Context,
    data_dict: dict[str, Any],
) -> dict[str, Any]:
    module = data_dict.get("module", "")

    tk.check_access("sysadmin", context, data_dict)

    if module:
        redis: Redis = connect_to_redis()

        redis_key: str = module + self_config.SELFINFO_REDIS_SUFFIX
        now: float = datetime.utcnow().timestamp()

        data: Mapping[str, Any] = {
            "name": module,
            "current_version": imetadata.version(module),
            "updated": now,
            "latest_version": selfutils.get_lib_latest_version(module),
        }

        for key in data:
            if data[key] != redis.hget(redis_key, key):
                redis.hset(redis_key, key=key, value=data[key])

        result: dict[str, Any] = {
            k.decode("utf-8"): v.decode("utf-8")
            for k, v in redis.hgetall(redis_key).items()
        }

        result["updated"] = datetime.fromtimestamp(float(result["updated"]))

        return result
    return {}
